#!/usr/bin/perl -w
#
# gluserpurge.pl by Usurper
# This is my first perl script, so if something is badly done, you know why.
# Email me with comments/suggestions/bug reports: usurper@home.com
# Run this with no parameters or with -h for a help screen.

$configpath = "/etc/glftpd.conf"; #default location of config file

$version = "1.03";   # May 2001

###### main()
$progname = $0;
getargs();
readconf();
$userpath = "$rootpath/$datapath/users";
$userpath =~ s/\/+/\//g;  # get rid of multiple slashes
if (defined($debugmode)) {
  print "Using rootpath $rootpath\n";
  print "Using datapath $datapath\n";
  print "Using userpath $userpath\n";
}

if (defined($listonly)) {
  print "Listing ";
} else {
  print "Deleting ";
}
print "users who haven't logged on in $seendays days\n\n";

$curtime = time;
$totaldeleted = 0;
dousers();
if ($totaldeleted > 0) {
  print "Deleted $totaldeleted users. Issue 'site purge' to purge them.\n";
}

###### end main()

sub printhelp {
  print "gluserpurge version $version by Usurper\n";
  print "\nUsage: $progname [-v] [-h] [-b] [-a] [-l] [-r path] -d numdays\n";
  print " numdays means delete users who haven't logged on in that many days\n";
  print " -a : delete all users who haven't been seen for <numdays> without ";
  print "asking\n      for confirmation (be careful with this).\n";
  print " -l : list inactive users only, don't delete anyone.\n";
  print " -r : alternate path to config file.\n";
  print " -v : print version number.\n";
  print " -b : deBug mode - prints some extra info.\n";
  print " -h : print this help screen.\n\n";
}

sub delete_user {
  my $count = 0;
  while (-f "$userpath/$userfile.lock" and $count <= 10) {
    if ($count eq 0) {
      print "Waiting for lockfile to disappear...\n";
    }
    $count++;
    if ($count > 10) {
      print "Removing stale lockfile for $userfile\n";
      unlink("$userpath/$userfile.lock") or
        die "Can't delete $userpath/$userfile.lock\n";
    } else {
      sleep(1);
    }
  }
  open(LOCKFILE, ">$userpath/$userfile.lock") or
    die "Can't create $userpath/$userfile.lock\n";
  print LOCKFILE "$$\n";
  close(LOCKFILE);
  open(OLDFILE, "$userpath/$userfile") or
    die "Can't open $userpath/$userfile for reading\n";
  open(NEWFILE, ">$userpath/$userfile.$$") or
    die "Can't create $userpath/$userfile.$$\n";
  while (defined($line=<OLDFILE>)) {
    if (substr($line, 0, 6) eq "FLAGS ") {
      my @line2 = split(' ', $line);
      if (!defined($line2[1])) {
        print NEWFILE "FLAGS 6\n";
      } else {
        print NEWFILE "FLAGS " . "$line2[1]" . "6\n";
      }
    } else {
      print NEWFILE "$line";
    }
  }
  close(OLDFILE);
  close(NEWFILE);
  unlink("$userpath/$userfile") or die "Can't delete $userfile\n";
  rename("$userpath/$userfile.$$", "$userpath/$userfile");
  unlink("$userpath/$userfile.lock");
  print "user $userfile deleted\n";
  $totaldeleted++;
}

sub getargs {
 while ($arg = shift(@ARGV)) {
  if (substr($arg, 0, 1) ne "-") {
    print "Invalid argument: $arg\n";
    next;
  }
  $arg = substr($arg, 1);
  if (substr($arg, 0, 1) eq "d") {
    if (length($arg) > 1) { # no space before argument
      $seendays = substr($arg, 1);
    } else { # space before argument
      $seendays = shift(@ARGV) or die "need argument to -d\n";
    }
    if ($seendays !~ /^[0-9]+$/ or $seendays <= 0 or $seendays > 10000) {
      print "Error: incorrect parameter to the -d option: $seendays\n";
      printhelp();
      exit 1;
    }
  } elsif (substr($arg, 0, 1) eq "r") {
    if (length($arg) > 1) {
      $configpath = substr($arg, 1);
    } else {
      $configpath = shift(@ARGV) or die "need argument to -r\n";
    }
  } elsif ($arg eq "h") {
    printhelp();
    exit 0;
  } elsif ($arg eq "a") {
    $deleteall = 1;
  } elsif ($arg eq "l") {
    $listonly = 1;
  } elsif ($arg eq "b") {
    $debugmode = 1;
  } elsif ($arg eq "v") {
    print "gluserpurge version $version by Usurper\n";
    exit 0;
  } else {
    print "-unknown option: $arg\n";
    printhelp();
    exit 1;
  }
 }
 if (!defined($seendays)) {
   printhelp();
   exit 0;
 }
 if (! -r $configpath or ! -f $configpath) {
   print "Error: can't read config file $configpath.\n";
   printhelp();
   exit 1;
 }
 if ($seendays < 10 and !defined($listonly)) {
   print "Are you sure you want to delete users who haven't been seen for only $seendays days? ";
   my $response = <STDIN>;
   chomp $response;
   $response = lc($response);
   if ($response ne "y" && $response ne "yes" && $response ne "yup" && $response ne "yeah") {
     die "Whew.\n";
   } else {
     print "Ok, whatever, it's your site.\n";
   }
 }
}

sub readconf {
  open(CONFFILE, $configpath) or die "can't open $configpath: $!";
  do {
    if (!defined($line = <CONFFILE>)) {
      die "rootpath or datapath not found in $configpath\n";
    }
    if (substr($line, 0, 8) eq "datapath") {
      $datapath = substr($line, 8);
    } elsif (substr($line, 0, 8) eq "rootpath") {
      $rootpath = substr($line, 8);
    }
  } while (!defined($rootpath) or !defined($datapath));
  close(CONFFILE);
  $rootpath =~ s/^\s+//; # removing whitespace from front
  $rootpath =~ s/\s+$//; # and back
  $datapath =~ s/^\s+//; # removing whitespace from front
  $datapath =~ s/\s+$//; # and back
}

sub dousers {
 opendir(DIR, $userpath) or die "can't open $userpath: $!";
 while (defined($userfile = readdir(DIR))) {
  next if (substr($userfile, 0, 8) eq "default.");
  next if (! -f "$userpath/$userfile");
  if (!open(FILE, "$userpath/$userfile")) {
    print "Can't open userfile $userpath/$userfile for reading: $!\n";
    next;
  }
  $histime = undef;
  while (defined($line = <FILE>) and !defined($histime)) {
    if (substr($line, 0, 6) eq "FLAGS ") {
      @line2 = split(' ', $line);
      next if (!defined($line2[1]));
      if ($line2[1] =~ m/6/) {
        print "Skipping deleted user $userfile.\n" if (defined($debugmode));
        $histime = "skip";
        last;
      } elsif ($line2[1] =~ m/8/) {
        print "Skipping anonymous user $userfile.\n" if (defined($debugmode));
        $histime = "skip";
        last;
      }
    }
    next if (substr($line, 0, 5) ne "TIME ");
    @line2 = split(' ',$line);
    $histime = $line2[2];
  }
  close(FILE);
  die "TIME line in file $userfile not found.\n" if (!defined($histime));

  next if ($histime eq "skip");
  $difftime = int(($curtime - $histime)/60/60/24);
  print "Inactive time for $userfile is $difftime days.\n"
    if (defined($debugmode));

  if ($difftime > $seendays) {  # user not seen recently
    if (defined($listonly)) {
      print "User $userfile inactive for $difftime days.\n";
      next;
    }
    if (defined($deleteall)) {
      print "Deleting $userfile...\n";
      delete_user();
      next;
    }
    my $response;
    while (1) {
      print "Delete user $userfile (inactive for $difftime days)? (y,N,a,v,q,?) ";
      $response = lc(substr(<STDIN>, 0, 1));
      if ($response eq "v") {
        system("more", "$userpath/$userfile");
        next;
      }
      last unless ($response eq "?");
      print "\nChoices: a = delete this user and all others who match\n";
      print "         y = delete this user\n";
      print "         n = keep this user (default)\n";
      print "         v = view this user's userfile\n";
      print "         q = quit\n";
      print "         ? = help\n";
    }
    if ($response eq "q") {
      return;
    }
    if ($response eq "a") {
      print "\nDeleting all.\n";
      $deleteall = 1;
      $response = "y";
    }
    if ($response eq "y") {
      print "Deleting $userfile...\n";
      delete_user();
    } else {
      print "Keeping $userfile\n";
    }
  } 
 }
 closedir(DIR);
}

